/* 11 bit message ID
  DATA TYPE  NODE ID 
  111111     11111
*/
#include <stm32f10x_gpio.h>
/*
The node IDs serve two purposes:?  
they allow receiving nodes to  know about the origin of the message, 
and they ensure  that there is never more than one message starting 
  on the bus with the same message ID. ?
*/
// ================ NODE IDs ==================
//5 bits from 0 0000  to 1 1111 (31d)
enum Node_IDs{
  ID_NOT_USED  = 0, //0 0000 
  ID_AUTOPILOT = 1, //0 0001
  ID_BLUETOOTH = 2, //0 0010
  ID_EEPROM    = 5,
  ID_FTDI      = 6,
  
  ID_IMU       = 9, 
  ID_LCD       = 12,
  
  ID_RCRADIO   = 18,
  ID_SERVO     = 19,
  ID_TELEMETRY = 20,
  ID_ULTRASONIC= 21,
  ID_WAVEPLAY  = 23, 
    
  ID_MOTOR0    = 24, //1 1000 
  ID_MOTOR1,         //1 1001
  ID_MOTOR2,         //1 1010
  ID_MOTOR3,         //1 1011 27d   
  ID_ALL_MOTORS, 
  
  ID_ALL_NODES =  0x1F //1 1111 31d
};
//The 4 motor IDs can start at any xxx00 boundary.
//The IDs are constructed at runtime from jumper settings
//so that the same code can be used for all motors nodes.
//The other motor IDs are built on the value of MOTOR0.
//Bit0 = PB12
//Bit1 = PB13
//00  01
//10  11

// ================DATA TYPES ==================
//6 bits from 10 0000 (32d) to 11 1111 (63d)
enum Data_Types{
  DATA_NOT_USED     =  32, //0x20, 10 0000
  DATA_MOTOR_CONTROL,
  DATA_MOTOR_SENSORS,
    
  DATA_IMU_ANGLES,
  DATA_IMU_GYROS,
  DATA_IMU_ACCELS,
  DATA_RANGE,
  DATA_TURRET_CONTROL,
    
  DATA_BT_TEXT = 40,
  DATA_AUTO_SCRIPT,
    
  DATA_ERROR_MSG=46,
  DATA_WAVE_PLAY,
  DATA_ID_MSG, 
  DATA_TEXT_MSG,
      
  DATA_INT16x4 = 53, 
  DATA_INT32x2,
  DATA_FLOATx2,
  DATA_HEXx8,
    
  DATA_RESERVED   = 63 //0x3F, 11 1111
}; 

void can_config(void);
void can_nvic_config(void);
void can_tx_message_init(CanTxMsg *message);

uint32_t can_rx_message_origin_id(uint32_t RxID);
uint32_t can_rx_message_data_type(uint32_t RxID);

uint8_t can_messages_available(void);
uint8_t can_read_message(uint32_t *data_type, uint32_t *origin_id, uint8_t message_data[]);
uint8_t can_send_message(uint32_t data_type, uint8_t message_data[]);

void can_filter_n_config(uint8_t FilterNumber, int idLo, int maskLo, int idHigh, int maskHigh);
void can_filter_data_type(uint8_t FilterNumber, int data_type1, int data_type2);
void can_filter_origin_id(uint8_t FilterNumber, int origin_id1, int origin_id2);

uint8_t scan_cmd_str(uint8_t command_string[], uint8_t *command_char, int16_t *value);


typedef struct can_motor_sensors_t
{ 
  int16_t dutycycle, encoder, adc, reserved;
}can_motors_sensors_t;

typedef struct can_motors_control_t
{ 
  int16_t forward, steer, motors_enable, reserved;
}can_motors_control_t;

typedef struct can_imu_angles_raw_t 
{ 
  int16_t roll, pitch, yaw, temp_degC;
}can_imu_angles_raw_t;

typedef union 
{
  can_imu_angles_raw_t imu_angles;
  can_motors_control_t motors_control;
  can_motors_sensors_t motor_sensors;
  uint8_t ui8buf[8];
  int16_t i16buf[4];
  int32_t i32buf[2];
  float f32buf[2];
} can_data_union_t;

